" AutoAlign.vim: a ftplugin for tex
" Author:	Charles E. Campbell, Jr.  <NdrOchip@ScampbellPfamily.AbizM>-NOSPAM
" Date:		Jul 02, 2004
" Version:	4
" History:
"   4 : * see |i_ctrl-g_u| -- breaks undo sequence at every align
"       * ve=all caused problems when backspacing over \ - fixed!
"   3 : * autoalign not taken if a no-pattern line is
"         in-between the keepalign line and the current line
"   2 : * b:autoalign==0: turns autoalign off
"         b:autoalign==1: turns autoalign back on
"       * 'a now used during autoalign, and AlignMap's \t=
"         If user changes 'a, then AutoAlign recognizes that
"         it is not to keep aligning
"       * The AA command can be used to toggle AutoAlign
"   1 : The Epoch
if exists("b:did_tex_autoalign")
 finish
endif
let b:did_tex_autoalign = 4

" ---------------------------------------------------------------------
" Public Interface: AA toggles AutoAlign {{{1
com! -nargs=0 AA if exists("b:autoalign")|let b:autoalign= !b:autoalign|else|let b:autoalign= 0|endif|echo "AutoAlign is ".(b:autoalign? "on" : "off")

" ---------------------------------------------------------------------
"  overloading '=' to keep things lined up {{{1
ino <silent> \\ \\<c-o>:let b:autoalign_vekeep=&ve<bar>set ve=<cr><c-o>:silent call <sid>KeepTEXAligned()<cr><c-o>:let &ve=b:autoalign_vekeep<cr>

fun! s:KeepTEXAligned()
  if exists("b:autoalign") && b:autoalign == 0
   return
  endif

  set lz
  if &ve == "all"
   norm l
  endif

  if getline(".") =~ '^\([^&]*&\)\+[^&]*\\\{2}'
   let curposn   = SaveWinPosn()
   let curline   = line(".")
   let nopatline = search('^.*\(\\\\\)\@<!$\&^.','bW')
   call RestoreWinPosn(curposn)
   if exists("b:keeptexaligned") && line("'a") == b:keeptexaligned && b:keeptexaligned < curline && nopatline < line("'a")
   	exe "norm! i\<c-g>u\<esc>"
   	norm \tt$
   else
   	let b:keeptexaligned= line(".")
	ka
   endif
  elseif exists("b:keeptexaligned")
   unlet b:keeptexaligned
  endif

  set nolz
endfun

" ---------------------------------------------------------------------
" vim: ts=4 fdm=marker
